@@ -892,7 +892,6 @@ make_message(struct dhcp_message **message,
 
 	if (lease->addr.s_addr && lease->cookie == htonl(MAGIC_COOKIE)) {
 		if (type == DHCP_DECLINE ||
-		    type == DHCP_DISCOVER ||
 		    (type == DHCP_REQUEST &&
 			lease->addr.s_addr != iface->addr.s_addr))
 		{
@@ -915,6 +914,9 @@ make_message(struct dhcp_message **message,
 		p += len;
 	}
 
+	if (type == DHCP_DISCOVER && ifo->options & DHCPCD_REQUEST)
+		PUTADDR(DHO_IPADDRESS, ifo->req_addr);
+
 	if (type == DHCP_DISCOVER ||
 	    type == DHCP_INFORM ||
 	    type == DHCP_REQUEST)
@@ -1052,7 +1054,7 @@ write_lease(const struct interface *iface, const struct dhcp_message *dhcp)
 	syslog(LOG_DEBUG, "%s: writing lease `%s'",
 	    iface->name, iface->leasefile);
 
-	fd = open(iface->leasefile, O_WRONLY | O_CREAT | O_TRUNC, 0400);
+	fd = open(iface->leasefile, O_WRONLY | O_CREAT | O_TRUNC, 0444);
 	if (fd == -1) {
 		syslog(LOG_ERR, "%s: open: %m", iface->name);
 		return -1;
