@@ -1,6 +1,6 @@
 /* 
  * dhcpcd - DHCP client daemon
- * Copyright (c) 2006-2010 Roy Marples <roy@marples.name>
+ * Copyright (c) 2006-2011 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -25,7 +25,7 @@
  * SUCH DAMAGE.
  */
 
-const char copyright[] = "Copyright (c) 2006-2010 Roy Marples";
+const char copyright[] = "Copyright (c) 2006-2011 Roy Marples";
 
 #include <sys/file.h>
 #include <sys/socket.h>
@@ -88,6 +88,7 @@ char **ifdv = NULL;
 
 static char **margv;
 static int margc;
+static struct if_options *if_options;
 static char **ifv;
 static int ifc;
 static char *cffile;
@@ -161,6 +162,8 @@ cleanup(void)
 	struct interface *iface;
 	int i;
 
+	free_options(if_options);
+
 	while (ifaces) {
 		iface = ifaces;
 		ifaces = iface->next;
@@ -806,17 +809,23 @@ configure_interface1(struct interface *iface)
 	if (ifo->options & DHCPCD_CLIENTID)
 		syslog(LOG_DEBUG, "%s: using ClientID %s", iface->name,
 		    hwaddr_ntoa(iface->clientid + 1, *iface->clientid));
+	else
+		syslog(LOG_DEBUG, "%s: using hwaddr %s", iface->name,
+		    hwaddr_ntoa(iface->hwaddr, iface->hwlen));
 }
 
 int
 select_profile(struct interface *iface, const char *profile)
 {
 	struct if_options *ifo;
+	int ret;
 
+	ret = 0;
 	ifo = read_config(cffile, iface->name, iface->ssid, profile);
 	if (ifo == NULL) {
 		syslog(LOG_DEBUG, "%s: no profile %s", iface->name, profile);
-		return -1;
+		ret = -1;
+		goto exit;
 	}
 	if (profile != NULL) {
 		strlcpy(iface->state->profile, profile,
@@ -827,8 +836,11 @@ select_profile(struct interface *iface, const char *profile)
 		*iface->state->profile = '\0';
 	free_options(iface->state->options);
 	iface->state->options = ifo;
-	configure_interface1(iface);
-	return 0;
+
+exit:
+	if (profile)
+		configure_interface1(iface);
+	return ret;
 }
 
 static void
@@ -838,7 +850,6 @@ start_fallback(void *arg)
 
 	iface = (struct interface *)arg;
 	select_profile(iface, iface->state->options->fallback);
-	configure_interface1(iface);
 	start_interface(iface);
 }
 
@@ -1088,6 +1099,7 @@ start_interface(void *arg)
 	struct stat st;
 	struct timeval now;
 	uint32_t l;
+	int nolease;
 
 	handle_carrier(iface->name);
 	if (iface->carrier == LINK_DOWN) {
@@ -1119,21 +1131,9 @@ start_interface(void *arg)
 		delete_timeout(NULL, iface);
 		return;
 	}
-	if (ifo->req_addr.s_addr) {
-		iface->state->offer =
-		    dhcp_message_new(&ifo->req_addr, &ifo->req_mask);
-		if (ifo->options & DHCPCD_REQUEST)
-			ifo->req_addr.s_addr = 0;
-		else {
-			iface->state->reason = "STATIC";
-			iface->state->new = iface->state->offer;
-			iface->state->offer = NULL;
-			get_lease(&iface->state->lease, iface->state->new);
-			configure(iface);
-			start_inform(iface);
-			return;
-		}
-	} else
+	/* We don't want to read the old lease if we NAK an old test */
+	nolease = iface->state->offer && options & DHCPCD_TEST;
+	if (!nolease)
 		iface->state->offer = read_lease(iface);
 	if (iface->state->offer) {
 		get_lease(&iface->state->lease, iface->state->offer);
@@ -1192,6 +1192,10 @@ init_state(struct interface *iface, int argc, char **argv)
 	configure_interface(iface, argc, argv);
 	if (!(options & DHCPCD_TEST))
 		run_script(iface);
+	/* We need to drop the leasefile so that start_interface
+	 * doesn't load it. */	
+	if (ifs->options->options & DHCPCD_REQUEST)
+		unlink(iface->leasefile);
 
 	if (ifs->options->options & DHCPCD_LINK) {
 		switch (carrier_status(iface)) {
@@ -1266,6 +1270,43 @@ handle_interface(int action, const char *ifname)
 	}
 }
 
+#ifdef RTM_CHGADDR
+void
+handle_hwaddr(const char *ifname, unsigned char *hwaddr, size_t hwlen)
+{
+	struct interface *ifp;
+	struct if_options *ifo;
+
+	for (ifp = ifaces; ifp; ifp = ifp->next)
+		if (strcmp(ifp->name, ifname) == 0 && ifp->hwlen <= hwlen) {
+			ifo = ifp->state->options;
+			if (!(ifo->options &
+			    (DHCPCD_INFORM | DHCPCD_STATIC | DHCPCD_CLIENTID))
+	    		    && ifp->state->new != NULL &&
+			    ifp->state->new->cookie == htonl(MAGIC_COOKIE))
+			{
+				syslog(LOG_INFO,
+				    "%s: expiring for new hardware address",
+				    ifp->name);
+				drop_config(ifp, "EXPIRE");
+			}
+			memcpy(ifp->hwaddr, hwaddr, hwlen);
+			ifp->hwlen = hwlen;
+			if (!(ifo->options &
+			    (DHCPCD_INFORM | DHCPCD_STATIC | DHCPCD_CLIENTID)))
+			{
+				syslog(LOG_DEBUG, "%s: using hwaddr %s",
+				    ifp->name,
+		    		    hwaddr_ntoa(ifp->hwaddr, ifp->hwlen));
+				ifp->state->interval = 0;
+				ifp->state->nakoff = 1;
+				start_interface(ifp);
+			}
+		}
+	free(hwaddr);
+}
+#endif
+
 void
 handle_ifa(int type, const char *ifname,
     struct in_addr *addr, struct in_addr *net, struct in_addr *dst)
@@ -1633,7 +1674,8 @@ open_sockets(struct interface *iface)
 	if (iface->udp_fd == -1 &&
 	    iface->addr.s_addr != 0 &&
 	    iface->state->new != NULL &&
-	    iface->state->new->cookie == htonl(MAGIC_COOKIE))
+	    (iface->state->new->cookie == htonl(MAGIC_COOKIE) ||
+	    iface->state->options->options & DHCPCD_INFORM))
 	{
 		if (open_udp_socket(iface) == -1 && errno != EADDRINUSE)
 			syslog(LOG_ERR, "%s: open_udp_socket: %m", iface->name);
@@ -1663,7 +1705,6 @@ close_sockets(struct interface *iface)
 int
 main(int argc, char **argv)
 {
-	struct if_options *ifo;
 	struct interface *iface;
 	int opt, oi = 0, signal_fd, sig = 0, i, control_fd;
 	size_t len;
@@ -1707,6 +1748,9 @@ main(int argc, char **argv)
 		case 'T':
 			i = 1;
 			break;
+		case 'U':
+			i = 2;
+			break;
 		case 'V':
 			print_options();
 			exit(EXIT_SUCCESS);
@@ -1718,16 +1762,20 @@ main(int argc, char **argv)
 
 	margv = argv;
 	margc = argc;
-	ifo = read_config(cffile, NULL, NULL, NULL);
-	opt = add_options(ifo, argc, argv);
+	if_options = read_config(cffile, NULL, NULL, NULL);
+	opt = add_options(if_options, argc, argv);
 	if (opt != 1) {
 		if (opt == 0)
 			usage();
 		exit(EXIT_FAILURE);
 	}
-	options = ifo->options;
+	options = if_options->options;
 	if (i != 0) {
-		options |= DHCPCD_TEST | DHCPCD_PERSISTENT;
+		if (i == 1)
+			options |= DHCPCD_TEST;
+		else
+			options |= DHCPCD_DUMPLEASE;
+		options |= DHCPCD_PERSISTENT;
 		options &= ~DHCPCD_DAEMONISE;
 	}
 	
@@ -1740,7 +1788,7 @@ main(int argc, char **argv)
 	if (options & DHCPCD_QUIET)
 		close(STDERR_FILENO);
 
-	if (!(options & DHCPCD_TEST)) {
+	if (!(options & (DHCPCD_TEST | DHCPCD_DUMPLEASE))) {
 		/* If we have any other args, we should run as a single dhcpcd
 		 *  instance for that interface. */
 		len = strlen(PIDFILE) + IF_NAMESIZE + 2;
@@ -1757,6 +1805,36 @@ main(int argc, char **argv)
 		syslog(LOG_ERR, "chdir `/': %m");
 	atexit(cleanup);
 
+	if (options & DHCPCD_DUMPLEASE) {
+		if (optind != argc - 1) {
+			syslog(LOG_ERR, "dumplease requires an interface");
+			exit(EXIT_FAILURE);
+		}
+		ifaces = iface = xzalloc(sizeof(*iface));
+		strlcpy(iface->name, argv[optind], sizeof(iface->name));
+		snprintf(iface->leasefile, sizeof(iface->leasefile),
+		    LEASEFILE, iface->name);
+		iface->state = xzalloc(sizeof(*iface->state));
+		iface->state->options = xzalloc(sizeof(*iface->state->options));
+		strlcpy(iface->state->options->script, if_options->script,
+		    sizeof(iface->state->options->script));
+		iface->state->new = read_lease(iface);
+		if (iface->state->new == NULL && errno == ENOENT) {
+			strlcpy(iface->leasefile, argv[optind],
+			    sizeof(iface->leasefile));
+			iface->state->new = read_lease(iface);
+		}
+		if (iface->state->new == NULL) {
+			if (errno == ENOENT)
+				syslog(LOG_ERR, "%s: no lease to dump",
+				    iface->name);
+			exit(EXIT_FAILURE);
+		}
+		iface->state->reason = "DUMP";
+		run_script(iface);
+		exit(EXIT_SUCCESS);
+	}
+
 	if (!(options & (DHCPCD_MASTER | DHCPCD_TEST))) {
 		control_fd = open_control();
 		if (control_fd != -1) {
@@ -1867,7 +1945,7 @@ main(int argc, char **argv)
 		syslog(LOG_ERR, "init_socket: %m");
 		exit(EXIT_FAILURE);
 	}
-	if (ifo->options & DHCPCD_LINK) {
+	if (if_options->options & DHCPCD_LINK) {
 		linkfd = open_link_socket();
 		if (linkfd == -1)
 			syslog(LOG_ERR, "open_link_socket: %m");
@@ -1939,14 +2017,15 @@ main(int argc, char **argv)
 		{
 			syslog(LOG_WARNING, "no interfaces have a carrier");
 			daemonise();
-		} else if (ifo->timeout > 0) {
+		} else if (if_options->timeout > 0) {
 			if (options & DHCPCD_IPV4LL)
 				options |= DHCPCD_TIMEOUT_IPV4LL;
-			add_timeout_sec(ifo->timeout, handle_exit_timeout,
-			    NULL);
+			add_timeout_sec(if_options->timeout,
+			    handle_exit_timeout, NULL);
 		}
 	}
-	free_options(ifo);
+	free_options(if_options);
+	if_options = NULL;
 
 	sort_interfaces();
 	for (iface = ifaces; iface; iface = iface->next)
