@@ -95,6 +95,7 @@ const struct option cf_options[] = {
 	{"require",         required_argument, NULL, 'Q'},
 	{"static",          required_argument, NULL, 'S'},
 	{"test",            no_argument,       NULL, 'T'},
+	{"dumplease",       no_argument,       NULL, 'U'},
 	{"variables",       no_argument,       NULL, 'V'},
 	{"whitelist",       required_argument, NULL, 'W'},
 	{"blacklist",       required_argument, NULL, 'X'},
@@ -336,7 +337,8 @@ parse_option(struct if_options *ifo, int opt, const char *arg)
 	case 'g': /* FALLTHROUGH */
 	case 'n': /* FALLTHROUGH */
 	case 'x': /* FALLTHROUGH */
-	case 'T': /* We need to handle non interface options */
+	case 'T': /* FALLTHROUGH */
+	case 'U': /* We need to handle non interface options */
 		break;
 	case 'b':
 		ifo->options |= DHCPCD_BACKGROUND;
@@ -418,14 +420,12 @@ parse_option(struct if_options *ifo, int opt, const char *arg)
 		ifo->options |= DHCPCD_QUIET;
 		break;
 	case 'r':
-		ifo->options |= DHCPCD_REQUEST;
 		if (parse_addr(&ifo->req_addr, NULL, arg) != 0)
 			return -1;
+		ifo->options |= DHCPCD_REQUEST;
 		ifo->req_mask.s_addr = 0;
 		break;
 	case 's':
-		ifo->options |= DHCPCD_INFORM | DHCPCD_PERSISTENT;
-		ifo->options &= ~(DHCPCD_ARP | DHCPCD_STATIC);
 		if (arg && *arg != '\0') {
 			if (parse_addr(&ifo->req_addr, &ifo->req_mask,
 				arg) != 0)
@@ -434,6 +434,8 @@ parse_option(struct if_options *ifo, int opt, const char *arg)
 			ifo->req_addr.s_addr = 0;
 			ifo->req_mask.s_addr = 0;
 		}
+		ifo->options |= DHCPCD_INFORM | DHCPCD_PERSISTENT;
+		ifo->options &= ~(DHCPCD_ARP | DHCPCD_STATIC);
 		break;
 	case 't':
 		ifo->timeout = atoint(arg);
